#!/usr/bin/perl

use strict;
use utf8;

use Getopt::Long;
use Pod::Usage;

binmode(STDIN, ':utf8');
binmode(STDOUT, ':utf8');
binmode(STDERR, ':utf8');

my $verbose;
my $help;
my $man;

my $systemFile;
my $GSFile;
my $run;

my %GSData;
my %systemData;
my $evalGroup;
my $evalRel;
my $matchingMode = 'exact';

my %measureResults;

my %relationScheme = (
    'DIT' => {'SOURCE' => 1, 'OSEE' => 1},
    'SUR' => {'OSEE' => 1, 'CIBLE' => 1},
    'MOD' => {'MODIFIEUR' => 1, 'OSEE' => 1},
    'NEG' => {'NEGATION' => 1, 'OSEE' => 1},
    );

my %groupeScheme = (
    'SOURCE' => 1,
    'CIBLE' => 1,
    'NEGATION' => 1,
    'MODIFIEUR' => 1,
    'OSEE' => 1
    );

my %OSEE = (
    'ACCORD' => 1,
    'AMOUR' => 1,
    'APAISEMENT' => 1,
    'COLERE' => 1,
    'DEPLAISIR' => 1,
    'DERANGEMENT' => 1,
    'DESACCORD' => 1,
    'DESTINATAIRE' => 1,
    'DEVALORISATION' => 1,
    'ENNUI' => 1,
    'INSATISFACTION' => 1,
    'INSTRUCTION_DEMANDE' => 1,
    'MEPRIS' => 1,
    'NEGATIF' => 1,
    'PEUR' => 1,
    'PLAISIR' => 1,
    'POSITIF' => 1,
    'SATISFACTION' => 1,
    'SURPRISE_NEGATIVE' => 1,
    'SURPRISE_POSITIVE' => 1,
    'TRISTESSE' => 1,
    'VALORISATION' => 1,
    );

my %modes = (
    'exact' => 1,
    'inexact' => 1,
    );

if (scalar(@ARGV) ==0) {
    $help = 1;
}

Getopt::Long::Configure ("bundling");

GetOptions('help|?'     => \$help,
	   'man|m'     => \$man,
	   'verbose|v+'     => \$verbose,
	   'systemfile|s=s'     => \$systemFile,
	   'GSfile|g=s'     => \$GSFile,
	   'run|r=s' => \$run,
	   'relation|R' => \$evalRel,
	   'group|G' => \$evalGroup,
	   'exact' => sub {$matchingMode = 'exact';},
	   'inexact' => sub {$matchingMode = 'inexact';},
    );

warn "Matching mode: $matchingMode\n";

pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;

&readFile(\%relationScheme, $GSFile, \%GSData, $verbose);
&readFile(\%relationScheme, $systemFile, \%systemData, $verbose);

if (defined $evalGroup) {
    &makeEvalGrp(\%GSData, \%systemData, \%measureResults, $run, $verbose, $matchingMode);
    &computeEvalMeasures(\%GSData, \%systemData, \%measureResults, "group", $verbose)
}
if (defined $evalRel) {
    &makeEvalRel(\%GSData, \%systemData, \%measureResults, $run, $verbose, $matchingMode);
    &computeEvalMeasures(\%GSData, \%systemData, \%measureResults, "relation", $verbose)
}
sub makeEvalGrp {
    my ($GSdata, $systmdata, $results, $r, $verb, $mode) = @_;

    my $id;
    my $i;
    my $dataG;
    my $dataS;
    my $i;
    my $group;
    
    warn "\nEvaluation of groups\n" if ($verb);
    foreach $id (keys %$GSdata) {
	warn "Process tweet $id:\n" if ($verb);
	if (exists $systmdata->{$id}) {
	    warn "  nb of groups in GS: " . scalar(@{$GSdata->{$id}->{group}}) . "\n" if ($verb);
	    warn "  nb of groups in System: " . scalar(@{$systmdata->{$id}->{group}}) . "\n" if ($verb);
	    foreach $dataG (@{$GSdata->{$id}->{group}}) {
		if ($dataG->{type} eq "group") {
		    warn "  group ($id): " . $dataG->{data}->{elt} . "\n" if ($verb >= 2);
		    $i = 0; # (!$systmdata->{$id}->{group}->[$i]->{seen})
		    while(($i < scalar(@{$systmdata->{$id}->{group}}))) {
			warn "    " . $systmdata->{$id}->{group}->[$i]->{data}->{elt} . "\n" if ($verb >= 2);
			if ((!$systmdata->{$id}->{group}->[$i]->{seen}) && (&matching($dataG->{data}, $systmdata->{$id}->{group}->[$i]->{data},$mode))) {
			    # TP
			    $systmdata->{$id}->{group}->[$i]->{seen} = 1;
			    $results->{group}->{doc}->{$id}->{doc}->{TP}++;
			    $results->{group}->{doc}->{$id}->{group}->{$dataG->{data}->{elt}}->{TP}++;

			    $results->{group}->{group}->{$dataG->{data}->{elt}}->{group}->{TP}++;

			    $results->{group}->{systm}->{doc}->{TP}++;
			    $results->{group}->{systm}->{group}->{$dataG->{data}->{elt}}->{TP}++;
			    last;
			}
			$i++;
		    }
		    if ($i == scalar(@{$systmdata->{$id}->{group}})) {
			# FN
			$results->{group}->{doc}->{$id}->{doc}->{FN}++;
			$results->{group}->{doc}->{$id}->{group}->{$dataG->{data}->{elt}}->{FN}++;

			$results->{group}->{group}->{$dataG->{data}->{elt}}->{group}->{FN}++;

			$results->{group}->{systm}->{doc}->{FN}++;
			$results->{group}->{systm}->{group}->{$dataG->{data}->{elt}}->{FN}++;
		    }
		}
	    }
	    foreach $dataS (@{$systmdata->{$id}->{group}}) {
		if (!$dataS->{seen}) {
		    # FP
		    $results->{group}->{doc}->{$id}->{doc}->{FP}++;
		    $results->{group}->{doc}->{$id}->{group}->{$dataS->{data}->{elt}}->{FP}++;

		    $results->{group}->{group}->{$dataS->{data}->{elt}}->{group}->{FP}++;

		    $results->{group}->{systm}->{doc}->{FP}++;
		    $results->{group}->{systm}->{group}->{$dataS->{data}->{elt}}->{FP}++;
		}
	    }
	} else {
	    $results->{group}->{doc}->{$id}->{doc}->{TP} = 0;
	    foreach $dataG (@{$GSdata->{$id}->{group}}) {
		if ($dataG->{type} eq "group") {
		    $results->{group}->{doc}->{$id}->{group}->{$dataG->{data}->{elt}}->{TP}=0;

		    $results->{group}->{doc}->{$id}->{doc}->{FN}++;
		    $results->{group}->{doc}->{$id}->{group}->{$dataG->{data}->{elt}}->{FN}++;

		    $results->{group}->{group}->{$dataG->{data}->{elt}}->{group}->{FN}++;
		    
		    $results->{group}->{systm}->{doc}->{FN}++;
		    $results->{group}->{systm}->{group}->{$dataG->{data}->{elt}}->{FN}++;
		}
	    }
	}
    }
    foreach $id (keys %$systmdata) {
	foreach $dataS (@{$systmdata->{$id}->{group}}) {
	    if ((!$dataS->{seen}) && (!exists $GSdata->{$id})) {
#	    if ((!$dataS->{seen}) && (!exists $GSdata->{$id}->{group}->{$dataS}))  {
		# FP
		$results->{group}->{doc}->{$id}->{doc}->{FP}++;
		$results->{group}->{doc}->{$id}->{group}->{$dataS->{data}->{group}}->{FP}++;
		
		$results->{group}->{group}->{$dataS->{data}->{group}}->{group}->{FP}++;
		
		$results->{group}->{systm}->{doc}->{FP}++;
		$results->{group}->{systm}->{group}->{$dataS->{data}->{group}}->{FP}++;
	    }
	}
    }
}

sub computeEvalMeasures {
    my ($GSdata, $systmdata, $results, $info, $verb) = @_;

    my $info2 = $info;
    my $id;
    my $group;
    my %prec;
    my %recall;
    my %fmeasure;
    my $p;
    my $r;
    my $f;

    foreach $id (keys %{$results->{$info}->{doc}}) {
	if ($verb) {
	    print "$id\n  Micro\n";
	}
	if ($verb >= 2) {
	    print "    TP: " . ($results->{$info}->{doc}->{$id}->{doc}->{TP} * 1) . "\n";
	    print "    FP: " . ($results->{$info}->{doc}->{$id}->{doc}->{FP} * 1) . "\n";
	    print "    FN: " . ($results->{$info}->{doc}->{$id}->{doc}->{FN} * 1) . "\n";
	}
	# && (scalar(@{$GSdata->{$id}->{$info2}}) > 0)
	# warn $GSdata->{$id}->{$info2} . "\n";
	if ((exists $GSdata->{$id}) && (exists $GSdata->{$id}->{$info2})&& (scalar(@{$GSdata->{$id}->{$info2}}) > 0)) {
	($p, $r, $f) = &computePrecRecallFMeasure($results->{$info}->{doc}->{$id}->{doc}->{TP}, $results->{$info}->{doc}->{$id}->{doc}->{FP}, $results->{$info}->{doc}->{$id}->{doc}->{FN});
	} else {
	    $p = 0;
	    $r = 0;
	    $f = 0;
	}
	$prec{$info}->{doc}->{$id}->{micro} = $p;
	$recall{$info}->{doc}->{$id}->{micro} = $r;
	$fmeasure{$info}->{doc}->{$id}->{micro} = $f;

	$prec{$info}->{systm}->{macro_doc} += $p;
	$recall{$info}->{systm}->{macro_doc} += $r;
	$fmeasure{$info}->{system}->{macro_doc} += $f;

	if ($verb) {
	print "    Precision: " . ($prec{$info}->{doc}->{$id}->{micro} * 1) . "\n";
	print "    Recall: " . ($recall{$info}->{doc}->{$id}->{micro} * 1) . "\n";
	print "    F-Measure1: " . ($fmeasure{$info}->{doc}->{$id}->{micro} * 1) . "\n";
	}
#	print "    Macro (class=group)\n";
	foreach $group (keys %{$results->{$info}->{doc}->{$id}->{$info2}}) {
	if ($verb) {
	    print "  group $group:\n";
	}
	    if ($verb >= 2) {
		print "\tTP: " . ($results->{$info}->{doc}->{$id}->{$info2}->{$group}->{TP} * 1) . "\n";
		print "\tFP: " . ($results->{$info}->{doc}->{$id}->{$info2}->{$group}->{FP} * 1) . "\n";
		print "\tFN: " . ($results->{$info}->{doc}->{$id}->{$info2}->{$group}->{FN} * 1) . "\n";
	    }
	    ($p, $r, $f) = &computePrecRecallFMeasure($results->{$info}->{doc}->{$id}->{$info2}->{$group}->{TP}, $results->{$info}->{doc}->{$id}->{$info2}->{$group}->{FP}, $results->{$info}->{doc}->{$id}->{$info2}->{$group}->{FN});
	    $prec{$info}->{doc}->{$id}->{$info2}->{$group} = $p;
	    $recall{$info}->{doc}->{$id}->{$info2}->{$group} = $r;
	    $fmeasure{$info}->{doc}->{$id}->{$info2}->{$group} = $f;

	if ($verb) {
	    print "    Precision: " . ($prec{$info}->{doc}->{$id}->{$info2}->{$group} * 1) . "\n";
	    print "    Recall: " . ($recall{$info}->{doc}->{$id}->{$info2}->{$group} * 1) . "\n";
	    print "    F-Measure1: " . ($fmeasure{$info}->{doc}->{$id}->{$info2}->{$group} * 1) . "\n";
	}
	    $prec{$info}->{doc}->{$id}->{macro_group} += $p;
	    $recall{$info}->{doc}->{$id}->{macro_group} += $r;
	    $fmeasure{$info}->{doc}->{$id}->{macro_group} += $f;
	}
	    # 
	if (scalar(keys %{$results->{$info}->{doc}->{$id}->{$info2}}) > 0) {
	    $prec{$info}->{doc}->{$id}->{macro_group} /= scalar(keys %{$results->{$info}->{doc}->{$id}->{$info2}});
	    $recall{$info}->{doc}->{$id}->{macro_group} /= scalar(keys %{$results->{$info}->{doc}->{$id}->{$info2}});
	    $fmeasure{$info}->{doc}->{$id}->{macro_group} /= scalar(keys %{$results->{$info}->{doc}->{$id}->{$info2}});
	} else {
#	    if (scalar(keys %{$results->{$info}->{doc}->{$id}->{$info2}}) == 0) {
	    if ((exists $GSdata->{$id}) && (scalar(@{$GSdata->{$id}->{$info2}}) == 0) &&
		(exists $systmdata->{$id}) && (scalar(@{$systmdata->{$id}->{$info2}}) == 0)) {
		$prec{$info}->{doc}->{$id}->{macro_group} = 1;
		$recall{$info}->{doc}->{$id}->{macro_group} = 1;
		$fmeasure{$info}->{doc}->{$id}->{macro_group} = 1;
	    } else {
		$prec{$info}->{doc}->{$id}->{macro_group} = 0;
		$recall{$info}->{doc}->{$id}->{macro_group} = 0;
		$fmeasure{$info}->{doc}->{$id}->{macro_group} = 0;
	    }
	}
	if ($verb) {
	print "  Macro (class=group)\n";
	print "    Precision: " . ($prec{$info}->{doc}->{$id}->{macro_group} * 1) . "\n";
	print "    Recall: " . ($recall{$info}->{doc}->{$id}->{macro_group} * 1) . "\n";
	print "    F-Measure1: " . ($fmeasure{$info}->{doc}->{$id}->{macro_group} * 1) . "\n";
	}
    }

    if (scalar(keys %{$results->{$info}->{doc}}) > 0) {
	$prec{$info}->{systm}->{macro_doc} /= scalar(keys %{$results->{$info}->{doc}});
	$recall{$info}->{systm}->{macro_doc} /= scalar(keys %{$results->{$info}->{doc}});
	$fmeasure{$info}->{system}->{macro_doc} /= scalar(keys %{$results->{$info}->{doc}});
    } else {
	if (scalar(keys %{$results->{$info}->{doc}}) == 0) {
#	if ((scalar(@{$GSdata->{$id}->{$info2}}) == 0) && (scalar(@{$systmdata->{$id}->{$info2}}) == 0)) {
	    $prec{$info}->{systm}->{macro_doc} = 1;
	    $recall{$info}->{systm}->{macro_doc} = 1;
	    $fmeasure{$info}->{system}->{macro_doc} = 1;
	} else {
	    $prec{$info}->{systm}->{macro_doc} = 0;
	    $recall{$info}->{systm}->{macro_doc} = 0;
	    $fmeasure{$info}->{system}->{macro_doc} = 0;
	}
    }
    
    if ($verb) {
    print "\n------------------------------\n";
	print "Micro\n";
    }
    if ($verb >= 2) {
	print "  TP: " . ($results->{$info}->{systm}->{doc}->{TP} * 1) . "\n";
	print "  FP: " . ($results->{$info}->{systm}->{doc}->{FP} * 1) . "\n";
	print "  FN: " . ($results->{$info}->{systm}->{doc}->{FN} * 1) . "\n";
    }
    ($p, $r, $f) = &computePrecRecallFMeasure($results->{$info}->{systm}->{doc}->{TP}, $results->{$info}->{systm}->{doc}->{FP}, $results->{$info}->{systm}->{doc}->{FN});
    $prec{$info}->{systm}->{micro} = $p;
    $recall{$info}->{systm}->{micro} = $r;
    $fmeasure{$info}->{system}->{micro} = $f;


	if ($verb) {
    print "\n";
	}
    foreach $group (keys %{$results->{$info2}->{$info2}}) {
    if ($verb) {
	print "  $group:\n";
    }
	if ($verb >= 2) {
	    print "\tTP: " . ($results->{$info2}->{$info2}->{$group}->{$info2}->{TP} * 1) . "\n";
	    print "\tFP: " . ($results->{$info2}->{$info2}->{$group}->{$info2}->{FP} * 1) . "\n";
	    print "\tFN: " . ($results->{$info2}->{$info2}->{$group}->{$info2}->{FN} * 1) . "\n";
	}
	($p, $r, $f) = &computePrecRecallFMeasure($results->{$info2}->{$info2}->{$group}->{$info2}->{TP}, $results->{$info2}->{$info2}->{$group}->{$info2}->{FP}, $results->{$info2}->{$info2}->{$group}->{$info2}->{FN});
	$prec{$info}->{$info2}->{$group} = $p;
	$recall{$info}->{$info2}->{$group} = $r;
	$fmeasure{$info}->{$info2}->{$group} = $f;

	$prec{$info}->{systm}->{macro_group} += $p;
	$recall{$info}->{systm}->{macro_group} += $r;
	$fmeasure{$info}->{system}->{macro_group} += $f;

    if ($verb) {
	print "    Precision: " . ($prec{$info}->{$info2}->{$group} * 1) . "\n";
	print "    Recall: " . ($recall{$info}->{$info2}->{$group} * 1) . "\n";
	print "    F-Measure1: " . ($fmeasure{$info}->{$info2}->{$group} * 1) . "\n";
    }
    }

    if (scalar(keys %{$results->{$info2}->{$info2}}) > 0) {
	$prec{$info}->{systm}->{macro_group} /= scalar(keys %{$results->{$info2}->{$info2}});
	$recall{$info}->{systm}->{macro_group} /= scalar(keys %{$results->{$info2}->{$info2}});
	$fmeasure{$info}->{system}->{macro_group} /= scalar(keys %{$results->{$info2}->{$info2}});
    } else {
	if (scalar((keys %{$results->{$info2}->{$info2}})) == 0) {
	    $prec{$info}->{systm}->{macro_group} = 1;
	    $recall{$info}->{systm}->{macro_group} = 1;
	    $fmeasure{$info}->{system}->{macro_group} = 1;
	} else {
	    $prec{$info}->{systm}->{macro_group} = 0;
	    $recall{$info}->{systm}->{macro_group} = 0;
	    $fmeasure{$info}->{system}->{macro_group} = 0;
	}
    }
    print "------------------------------\n";
    print "Micro\n";
    print "  Precision: " . ($prec{$info}->{systm}->{micro} * 1) . "\n";
    print "  Recall: " . ($recall{$info}->{systm}->{micro} * 1) . "\n";
    print "  F-Measure1: " . ($fmeasure{$info}->{system}->{micro} * 1) . "\n";

    if ($verb) {
	print "Macro (class=group)\n";
	print "  Precision: " . ($prec{$info}->{systm}->{macro_group} * 1) . "\n";
	print "  Recall: " . ($recall{$info}->{systm}->{macro_group} * 1) . "\n";
	print "  F-Measure1: " . ($fmeasure{$info}->{system}->{macro_group} * 1) . "\n";
    }
    print "Macro (class=doc)\n";
    print "  Precision: " . ($prec{$info}->{systm}->{macro_doc} * 1) . "\n";
    print "  Recall: " . ($recall{$info}->{systm}->{macro_doc} * 1) . "\n";
    print "  F-Measure1: " . ($fmeasure{$info}->{system}->{macro_doc} * 1) . "\n";

}

sub computePrecRecallFMeasure {
    my ($TP, $FP, $FN) = @_;

    my $prec = 0;
    my $recall = 0;
    my $fmeasure = 0;

    if (($TP == 0) && ($FP == 0) && ($FN ==0)) {
    	$prec = 1;
    	$recall = 1;
    	$fmeasure = 1;
    } else {

    if (($TP != 0) || ($FP !=0)) {
	$prec = $TP / ($TP + $FP);
    }
    if (($TP != 0) || ($FN !=0)) {
	$recall = $TP / ($TP + $FN);
    }
    if (($prec != 0) && ($recall != 0)) {
	$fmeasure = (2 * $prec * $recall) / ($prec+$recall);
    }
    }
    return($prec, $recall, $fmeasure);
}

sub matching {
    my ($elt1, $elt2, $mode) = @_;

    my $res = 0;
    if ($elt1->{elt} eq $elt2->{elt}) {
	return(&offsetMatching($elt1->{offset}, $elt2->{offset}, $mode));
    } else {
	return(0);
    }

}

sub matchingRel {
    my ($elt1, $elt2, $mode) = @_;

    my $res = 0;
    if (($elt1->{elt1} eq $elt2->{elt1}) && ($elt1->{elt2} eq $elt2->{elt2})) {
	return(&offsetMatching($elt1->{offset1}, $elt2->{offset1}, $mode) && &offsetMatching($elt1->{offset2}, $elt2->{offset2}, $mode));
    } else {
	return(0);
    }

}

sub offsetMatching {
    my ($offset1, $offset2, $mode) = @_;

    if ($mode eq "exact") {
	if (($offset1->[0] == $offset2->[0]) && 
	    ($offset1->[1] == $offset2->[1])) {
	    return(1);
	} 
    } elsif ($mode eq "inexact") {
	if (($offset2->[0] >= $offset1->[1]) || ($offset1->[0] >= $offset2->[1])) {
	    return(0);
	} else {
	    return(1);
	}
    }
    return(0);
}



sub makeEvalRel {
    my ($GSdata, $systmdata, $results, $r, $verb, $mode) = @_;

    my $id;
    my $i;
    my $dataG;
    my $dataS;

    warn "\nEvaluation of relations\n" if ($verb);
    $results->{relation}->{relation} = {};
    foreach $id (keys %$GSdata) {
	warn "Process tweet $id:\n" if ($verb);
	if (exists $systmdata->{$id}) {
	    warn "  nb of relations in GS: " . scalar(@{$GSdata->{$id}->{relation}}) . "\n" if ($verb);
	    warn "  nb of relations in Systm: " . scalar(@{$systmdata->{$id}->{relation}}) . "\n" if ($verb);
	    foreach $dataG (@{$GSdata->{$id}->{relation}}) {
		$results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{TP}=0;
		$results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{FP}=0;
		$results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{FN}=0;
		if ($dataG->{type} eq "relation") {
#		    warn "relation ($id)\n" if $verb;
		    # warn $systmdata->{$id} . "\n";
		    # foreach $dataS (@{$systmdata->{$id}->{relation}}) {
		    # 	if ($dataS->{type} eq "relation") {
		    warn "  relation ($id): " . $dataG->{data}->{relation} if ($verb >= 2);
		    
		    $i = 0; # (!$systmdata->{$id}->{relation}->[$i]->{seen})
		    while(($i < scalar(@{$systmdata->{$id}->{relation}}))) {
			if ((!$systmdata->{$id}->{relation}->[$i]->{seen}) && (&matchingRel($dataG->{data}, $systmdata->{$id}->{relation}->[$i]->{data},$mode))) {
			warn "    relation: " . $systmdata->{$id}->{relation}->[$i]->{data}->{relation} . "\n" if ($verb >= 2);
			    # TP
			    $systmdata->{$id}->{relation}->[$i]->{seen} = 1;
			    $results->{relation}->{doc}->{$id}->{doc}->{TP}++;
			    $results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{TP}++;
			    
			    $results->{relation}->{relation}->{$dataG->{data}->{relation}}->{relation}->{TP}++;
			    
			    $results->{relation}->{systm}->{doc}->{TP}++;
			    $results->{relation}->{systm}->{relation}->{$dataG->{data}->{relation}}->{TP}++;
			    last;
			}
			$i++;
		    }
		    if ($i == scalar(@{$systmdata->{$id}->{relation}})) {
#			warn "not found\n";
			# FN
			$results->{relation}->{doc}->{$id}->{doc}->{FN}++;
			$results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{FN}++;
			
			$results->{relation}->{relation}->{$dataG->{data}->{relation}}->{relation}->{FN}++;
			
			$results->{relation}->{systm}->{doc}->{FN}++;
			$results->{relation}->{systm}->{relation}->{$dataG->{data}->{relation}}->{FN}++;
		    }
		}
	    }
	    foreach $dataS (@{$systmdata->{$id}->{relation}}) {
		if (!$dataS->{seen}) {
		    # FP
		    $results->{relation}->{doc}->{$id}->{doc}->{FP}++;
		    $results->{relation}->{doc}->{$id}->{relation}->{$dataS->{data}->{relation}}->{FP}++;
		    
		    $results->{relation}->{relation}->{$dataS->{data}->{relation}}->{relation}->{FP}++;
		    
		    $results->{relation}->{systm}->{doc}->{FP}++;
		    $results->{relation}->{systm}->{relation}->{$dataS->{data}->{relation}}->{FP}++;
		}
	    }
	} else {
	    $results->{relation}->{doc}->{$id}->{doc}->{TP} = 0;
#	    $results->{relation}->{systm}->{doc}->{TP}
	    foreach $dataG (@{$GSdata->{$id}->{relation}}) {
		if ($dataG->{type} eq "relation") {
		    $results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{TP}=0;
#		    $results->{relation}->{systm}->{relation}->{$dataG->{data}->{relation}}->{TP} = 0;
		    
		    $results->{relation}->{doc}->{$id}->{doc}->{FN}++;
		    $results->{relation}->{doc}->{$id}->{relation}->{$dataG->{data}->{relation}}->{FN}++;
		    
		    $results->{relation}->{relation}->{$dataG->{data}->{relation}}->{relation}->{FN}++;
		    
		    $results->{relation}->{systm}->{doc}->{FN}++;
		    $results->{relation}->{systm}->{relation}->{$dataG->{data}->{relation}}->{FN}++;
		}
	    }
	}
    }
    foreach $id (keys %$systmdata) {
	foreach $dataS (@{$systmdata->{$id}->{relation}}) {
	    if ((!$dataS->{seen}) && (!exists $GSdata->{$id})) {
		# FP
		# if (($GSdata->{$id}) && (exists $GSdata->{$id}->{relation})) {
		#     warn "=> ". $GSdata->{$id}->{relation} . "\n";
		# }
		$results->{relation}->{doc}->{$id}->{doc}->{FP}++;
		$results->{relation}->{doc}->{$id}->{relation}->{$dataS->{data}->{relation}}->{FP}++;
		
		$results->{relation}->{relation}->{$dataS->{data}->{relation}}->{relation}->{FP}++;
		
		$results->{relation}->{systm}->{doc}->{FP}++;
		$results->{relation}->{systm}->{relation}->{$dataS->{data}->{relation}}->{FP}++;
	    }
	}
    }
}

sub readFile {
    my ($relnSchm, $file, $data, $verb) = @_;
    my @infos;
    my $line;
    my $type;

    open FILE, $file or die "No such file $file\n";
    while($line = <FILE>) {
	chomp $line;
	if ($line !~ /^\s*$/) {
	    @infos = split /\|/, $line;
	    if (!exists $data->{$infos[0]}) {
		$data->{$infos[0]}->{group} = [];
		$data->{$infos[0]}->{relation} = [];
	    }
	    my %infosH;
	    if (scalar(@infos) == 6) {
		$type = 'relation';
		%infosH = (
		    'docId' => $infos[0],
		    'relation' => $infos[1],
		    $infos[2] => [split(/-/, $infos[3])],
		    'elt1' => $infos[2],
		    'offset1' => [split(/-/, $infos[3])],
		    $infos[4] => [split(/-/, $infos[5])],
		    'elt2' => $infos[4],
		    'offset2' => [split(/-/, $infos[5])],
		);
		if (exists $relnSchm->{$infosH{'relation'}}) {
#		    warn $infosH{'elt1'} . " / " . $infosH{'elt2'} . "\n";
		    if (! exists $relnSchm->{$infosH{'relation'}}->{$infosH{'elt1'}}) {
			warn "*** erroneous element in relation " . $infosH{'relation'} . ": " . $infosH{'elt1'} . "(ignored)\n";
			
		    } elsif (! exists $relnSchm->{$infosH{'relation'}}->{$infosH{'elt2'}}) {
			warn "*** erroneous element in relation " . $infosH{'relation'} . ": " .  $infosH{'elt2'} . "(ignored)\n";

		    } else {
			push @{$data->{$infos[0]}->{relation}}, {
			    type => $type,
			    data => \%infosH, 
			    seen => 0
			}; 
		    }
		} else {
		    warn "*** relation " . $infosH{'relation'} . "not exists (ignored)\n";
		}
	    } else {
		$type = 'group';
		%infosH = (
		    'docId' => $infos[0],
		    'group' => $infos[1],
		    'elt' => $infos[2],
		    'offset' => [split(/-/, $infos[3])],
		    $infos[2] => [split(/-/, $infos[3])],
		    );
		push @{$data->{$infos[0]}->{group}}, {
		    type => $type,
		    data => \%infosH, 
		    seen => 0
		};
	    }
	}
    }
    close FILE;
    if ($verb) {
	warn "$file: " . scalar(keys(%$data)) . " lines read\n";
    }
}


########################################################################

=head1 NAME

evalT3.pl - Evaluation script for the tasks 3 of DEFT2015


=head1 SYNOPSIS

evalT3.pl [option] --systemfile <filename> --GSfile <filename> --run <string>

where option can be --help --man --verbose --relation --group --exact --inexact

=head1 OPTIONS AND ARGUMENTS

=over 4

=item --systemfile <filename>,  -s <filename>

Specification of the file containing the results issued from the participant system.

=item --GSfile <filename>, -g <filename>

Specification of the file containing the Gold Standard.

=item --run <string>, -r <string>

To specify the name of the run.

=item --relation, -R

This option leads to evaluate the relations.

=item --group, -G

This option leads to evaluate the groups.

=item --exact

This option allow to perform an exact match on the offset: as system
data which has exactly the same offsets as the gold standard data, is
consider as correct.

This option is exclusive with the option C<--inexact>

=item --inexact

For performing an inexact match on the offset: an annotation which
overlaps another is considered as correct.

This option allow to perform an exact match on the offset: as system
data whose offsets overlap offsets of the gold standard data, is
consider as correct.

This option is exclusive with the option C<--exact>


=item --help, -?

Print help message for using evalT3.pl

=item --man, -m

Print man page of evalT3.pl

=item --verbose, -v

Go into the verbose mode. Several uses of the option increase the
verbosity.

=back

=head1 DESCRIPTION


This script computes the results for the task 3 of DEFT 2015. It
provides micro and macro-precision, recall and f-measure according
the documents. Additional information is provided for each document
and per group or relation.

=head1 EXAMPLES

Evaluation on the task 3 of the groups for the run1 (file C<T3-2.txt>)
against the gold standard, with an exact matching of the offset

  Programs/evalT3.pl -g Test/T3-1.txt -s Test/T3-2.txt -r run1 -G --exact 

Evaluation on the task 3 of the groups for the run2 (file C<T3-3.txt>)
against the gold standard, with an exact matching of the offset, in a
light verbose mode

  Programs/evalT3.pl -g Test/T3-1.txt -s Test/T3-3.txt -r run2 -G --exact -v

Evaluation on the task 3 of the groups for the run2 (file C<T3-3.txt>)
against the gold standard, with an inexact matching of the offset

  Programs/evalT3.pl -g Test/T3-1.txt -s Test/T3-3.txt -r run2 -G --inexact 

Evaluation on the task 3 of the relations for the run1 (file C<T3-2.txt>)
against the gold standard, with an exact matching of the offset

  Programs/evalT3.pl -g Test/T3-1.txt -s Test/T3-2.txt -r run1 -R --exact 

Evaluation on the task 3 of the relations for the run1 (file
C<T3-2.txt>) against the gold standard, with an inexact matching of
the offset

  Programs/evalT3.pl -g Test/T3-1.txt -s Test/T3-2.txt -r run1 -R --inexact 

Evaluation on the task 3 of the relations for the run2 (file
C<T3-3.txt>) against the gold standard, with an exact matching of the
offset, in a strong verbose mode

  Programs/evalT3.pl -g Test/T3-1.txt -s Test/T3-3.txt -r run1 -R --exact -vv

=head1 SEE ALSO

DEFT 2015 web site: <https://deft.limsi.fr/2015/>

=head1 AUTHOR
g
Thierry Hamon, E<lt>hamon@limsi.frE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2015 Thierry Hamon

This is free software; you can redistribute it and/or modify it under
the same terms as Perl itself, either Perl version 5.8.4 or, at your
option, any later version of Perl 5 you may have available.

=cut

